//
//  pie-chart.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-17.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//


function PieChart(chartDiv, svg, width, height) {
    this.chartDiv = chartDiv;
    chartDiv.classed("pie-chart", true);
    this.svg = svg;

    this.dataFunction = function() { return []; };

    var radius = Math.min(width, height) / 2;
    this.arc = d3.svg.arc().innerRadius(0).outerRadius(radius);
    this.pieLayout = d3.layout.pie().value(function(d) { return d[1]; });
    this.color = d3.scale.category10();

    this.centeredContainer =
        svg
        .append("g")
        .classed("pie-chart", true)
        .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");
}


PieChart.prototype.update = function() {
    var self = this;
    var data = this.pieLayout(this.dataFunction());
    // TODO: Need data key so animations are correct when segments come and go

    var segments = this.centeredContainer
        .selectAll("g.arc")
        .data(data);

    // insert
    var enteredSegments =
        segments
        .enter()
        .append("g")
        .classed("arc", true);

    enteredSegments
        .append("path")
        .attr("fill", function(d, i) { return self.color(i); });

    // TODO: Put text last so it's not occluded by other segments
    enteredSegments
        .append("text")
        .attr("text-anchor", "middle")
        .attr("dy", ".35em") // fudge vertical centering
        .call(updateText);


    // delete
    segments
        .exit()
        .remove();


    // update (& insert)
    segmentTransitions = 
        segments
        .transition()
        .duration(750);

    segmentTransitions
        .select("path")
        .attrTween("d", arcTween);

    segmentTransitions
        .select("text")
        .call(updateText);


    function updateText(selection) {
        selection
            .text(function(d) { return d.data[0]; })
            .attr("transform", function(d) { return "translate(" + self.arc.centroid(d) + ")"; })
            .attr("opacity", function(d) {
                return Math.abs(d.endAngle - d.startAngle) > .15 ? 1 : 0;
            });
    }

    function arcTween(d) {
        if (this._current === undefined)
            this._current = d;

        var i = d3.interpolate(this._current, d);
        this._current = i(0);
        return function(t) {
            return self.arc(i(t));
        };
    };
}
