//
//  top-list.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-17.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//

var TopList_unique = 1;


function TopList(chartDiv, svg, width, height) {
    this.chartDiv = chartDiv;
    chartDiv.classed("top-list", true);
    this.svg = svg;
    this.width = width;
    this.height = height;

    this.count = 5;
    this.barColor = "white";
    this.dataFunction = function() { return []; };
    this.format = d3.format(".2p");

    this.textMargin = 6;
    this.textClippingID = "top-list-clip-" + TopList_unique++;
    this.svg
        .append("clipPath")
        .attr("id", this.textClippingID)
        .append("rect")
        .attr("x", this.textMargin)
        .attr("width", this.width - this.textMargin - 40 /* space for percentage */);

    this.xScale = d3.scale.linear().rangeRound([0, width]);
}


TopList.prototype.update = function() {
    var self = this;
    var clickHandler = this.clickHandler;

    var valuesAndProportionsDescending = this.dataFunction().sort(function(d1, d2) { return d2[1] - d1[1]; });
    var topValuesAndProportions = [];
    for (var i = 0; i < valuesAndProportionsDescending.length; i++) {
        if (topValuesAndProportions.length >= this.count)
            break;

        var valueAndProportion = valuesAndProportionsDescending[i];
        if (valueAndProportion[0] && valueAndProportion[1] > 0) // skip empty items
            topValuesAndProportions.push(valueAndProportion);
    }

    if (topValuesAndProportions.length > 0)
        this.xScale.domain([0, topValuesAndProportions[0][1]]);

    var heightPerItem = Math.floor(this.height / this.count);
    var barPadding = 2;

    this.svg
        .select("#" + this.textClippingID + " rect")
        .attr("height", heightPerItem);

    var list =
        this.svg
        .selectAll("g")
        .data(topValuesAndProportions, function(d) { return d[0]; });


    // insert
    var enteredGs =
        list
        .enter()
        .append("g")
        .classed("top-list", true)
        .attr("opacity", 0);

    enteredGs
        .append("rect")
        .attr("fill", this.barColor)
        .attr("x", 0).attr("y", barPadding)
        .attr("height", heightPerItem - 2 * barPadding);

    enteredGs
        .append("line")
        .attr("stroke", this.barColor)
        .attr("x1", 0)
        .attr("x2", this.width)
        .attr("y1", heightPerItem - barPadding + 0.5)
        .attr("y2", heightPerItem - barPadding + 0.5);

    enteredGs
        .append("text")
        .classed("name", true)
        .attr("dy", "0.25em")
        .attr("x", this.textMargin)
        .attr("y", heightPerItem / 2)
        .attr("clip-path", "url(#" + this.textClippingID + ")");

    enteredGs
        .append("text")
        .classed("percent", true)
        .attr("text-anchor", "end")
        .attr("dy", "0.25em")
        .attr("x", this.width - this.textMargin).attr("y", heightPerItem / 2);

    enteredGs
        .call(updatePosition);

    // clickable area
    enteredGs
        .append("rect")
        .attr("opacity", 0)
        .attr("x", 0).attr("y", barPadding)
        .attr("width", this.width)
        .attr("height", heightPerItem - 2 * barPadding)
        .classed("clickable", !!clickHandler)
        .on("click", function(d) {
            if (clickHandler)
                clickHandler(d[0]);
        });


    function updatePosition(selection) {
        selection
            .attr("transform", function(d, i) { return "translate(0, " + (heightPerItem * i) + ")"; })
    }


    // delete
    list
        .exit()
        .transition()
        .duration(250)
        .attr("opacity", 0)
        .remove();


    // update (& insert)
    list
        .transition()
        .duration(500)
        .call(updatePosition)
        .attr("opacity", 1)
        .select("rect")
        .delay(150)
        .attr("width", function(d) { return self.xScale(d[1]); });

    list
        .select("text.name")
        .text(function(d, i) { return "" + (i+1) + ". " + d[0]; });

    list
        .select("text.percent")
        .transition()
        .duration(500)
        .ease(d3.ease("linear"))
        .tween("text", function(d) {
            if (this._current === undefined)
                this._current = d[1];

            var i = d3.interpolate(this._current, d[1]);
            this._current = i(1);
            return function(t) {
                this.textContent = self.format(i(t));
            };
        });
};
